"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = exports.Builder = exports.ProgressEvent = void 0;
const events_1 = require("events");
const fs = require("fs");
const globby = require("globby");
// eslint-disable-next-line node/no-extraneous-import
const google_auth_library_1 = require("google-auth-library");
const googleapis_1 = require("googleapis");
const path = require("path");
const stream_1 = require("stream");
const tar = require("tar");
const util_1 = require("util");
const config_1 = require("./config");
const readFile = util_1.promisify(fs.readFile);
var ProgressEvent;
(function (ProgressEvent) {
    ProgressEvent["CREATING_BUCKET"] = "CREATING_BUCKET";
    ProgressEvent["UPLOADING"] = "UPLOADING";
    ProgressEvent["BUILDING"] = "BUILDING";
    ProgressEvent["COMPLETE"] = "COMPLETE";
    ProgressEvent["LOG"] = "LOG";
})(ProgressEvent = exports.ProgressEvent || (exports.ProgressEvent = {}));
/**
 * Class that provides the `deploy` method.
 */
class Builder extends events_1.EventEmitter {
    constructor(options = {}) {
        super();
        this.gcb = googleapis_1.google.cloudbuild('v1');
        this.gcs = googleapis_1.google.storage('v1');
        this.tag = options.tag;
        this.sourcePath = options.sourcePath || process.cwd();
        this.configPath = options.configPath; // || path.join(this.sourcePath, 'cloudbuild.yaml');
        options.scopes = ['https://www.googleapis.com/auth/cloud-platform'];
        this._auth = new google_auth_library_1.GoogleAuth(options);
    }
    /**
     * Deploy the current application using the given opts.
     */
    async build() {
        const auth = await this._auth.getClient();
        googleapis_1.google.options({ auth });
        this.emit(ProgressEvent.UPLOADING);
        const { bucket, file } = await this.upload();
        this.emit(ProgressEvent.BUILDING);
        const projectId = await this._auth.getProjectId();
        // load configuration
        const requestBody = await config_1.getConfig({
            configPath: this.configPath,
            sourcePath: this.sourcePath,
            projectId,
            tag: this.tag,
        });
        requestBody.source = { storageSource: { bucket, object: file } };
        // create the request to perform a build
        const res = await this.gcb.projects.builds.create({
            projectId,
            requestBody,
        });
        const result = res.data;
        // poll the operation until complete
        const operationId = res.data.name;
        try {
            await this.poll(operationId);
        }
        catch (e) {
            let log;
            try {
                log = await this.fetchLog(result);
            }
            catch (e) {
                // 🤷‍♂️
            }
            e.log = log;
            throw e;
        }
        // Log streaming is super hard to understand.  For now, just fetch the
        // log from a well known location *after* it's complete.
        // TODO: make it stream
        const log = await this.fetchLog(result);
        result.log = log;
        this.emit(ProgressEvent.COMPLETE);
        return result;
    }
    /**
     * Obtain the full text of the log after the build is complete.
     * At some point this should be replaced with streaming logs.
     * @param result The BuildResult returned from the create operation
     */
    async fetchLog(result) {
        const build = result.metadata.build;
        const logsBucket = build.logsBucket.split('gs://').filter(x => !!x)[0];
        const logFilename = `log-${build.id}.txt`;
        const logRes = await this.gcs.objects.get({
            bucket: logsBucket,
            object: logFilename,
            alt: 'media',
        });
        this.emit(ProgressEvent.LOG, logRes.data);
        return logRes.data;
    }
    /**
     * Given an operation, poll it until complete.
     * @private
     * @param name Fully qualified name of the operation.
     */
    async poll(name) {
        const res = await this.gcb.operations.get({ name });
        const operation = res.data;
        if (operation.error) {
            const message = JSON.stringify(operation.error);
            throw new Error(message);
        }
        if (operation.done) {
            return;
        }
        await new Promise(r => setTimeout(r, 3000));
        await this.poll(name);
    }
    /**
     * Upload a local file to GCS given a signed url
     * @private
     * @param localPath Fully qualified path to the zip on disk.
     * @param remotePath Signed url used to put the file to
     */
    async upload() {
        // check to see if the bucket exists
        const projectId = await this._auth.getProjectId();
        const bucketName = `${projectId}-gcb-staging-bbq`;
        const exists = await this.gcs.buckets.get({ bucket: bucketName }).then(() => true, () => false);
        // if it does not exist, create it!
        if (!exists) {
            this.emit(ProgressEvent.CREATING_BUCKET, bucketName);
            await this.gcs.buckets.insert({
                project: projectId,
                requestBody: {
                    name: bucketName,
                    lifecycle: {
                        rule: [{ action: { type: 'Delete' }, condition: { age: 1 } }],
                    },
                },
            });
        }
        // Get the full list of files that don't match .gcloudignore
        const ignorePatterns = await this.getIgnoreRules();
        const files = await globby('**/**', {
            ignore: ignorePatterns,
            cwd: this.sourcePath,
        });
        // create a tar stream with all the files
        const tarStream = tar.c({ gzip: true, cwd: this.sourcePath }, files);
        // There is a bizarre bug with node-tar where the stream it hands back
        // looks like a stream and talks like a stream, but it ain't a real
        // stream.  Pass it through a Duplex to make node-fetch happy.
        const bodyStream = new stream_1.PassThrough();
        tarStream.pipe(bodyStream);
        // upload the object via stream to GCS
        const file = Date.now().toString() + '.tar.gz';
        await this.gcs.objects.insert({
            bucket: bucketName,
            name: file,
            media: { mediaType: 'application/gzip', body: bodyStream },
        });
        return { bucket: bucketName, file };
    }
    /**
     * Look in the CWD for a `.gcloudignore` file.  If one is present, parse it,
     * and return the ignore rules as an array of strings.
     * @private
     */
    async getIgnoreRules() {
        const ignoreFile = path.join(this.sourcePath, '.gcloudignore');
        let ignoreRules = new Array();
        try {
            const contents = await readFile(ignoreFile, 'utf8');
            ignoreRules = contents.split('\n').filter(line => {
                return !line.startsWith('#') && line.trim() !== '';
            });
        }
        catch (e) {
            // 🤷‍♂️
        }
        return ignoreRules;
    }
}
exports.Builder = Builder;
async function build(options) {
    const builder = new Builder(options);
    return builder.build();
}
exports.build = build;
//# sourceMappingURL=index.js.map